#ifndef vdp_h
#define vdp_h

//#define COUNT_ALL_FRAMES

class Vdp : Mmio {
	_System* _sys;
	Cpu& cpu;
	Bus& bus;

	void (Vdp::*name_address_select)(u8 col);
	void (Vdp::*pattern_ram_load)();
	void (Vdp::*name_ram_load)();
	void (Vdp::*color_address_select)();
	void (Vdp::*pattern_address_select)(u8 col);
	void (Vdp::*color_ram_load)(u8 col);
	void (Vdp::*spritePreProcessing)();
	void (Vdp::*calcSprite)(u8 pos);
	void (Vdp::*checkSpriteOverflow)();
	u8 (Vdp::*resetStatusReg)();
	u8 (Vdp::*getBackdrop)();
	Color::Mode (Vdp::*getMode)();

	u8* frameBuffer;
	u8* linePtr;
	u16 vLine;

	u16 lineBufferCounter;
	bool frameComplete;

	unsigned frames_executed, frame_count;
	time_t prev_t, curr_t;
	u64 frameCounter;

	#define BACKDROP_COLOR (0x10 | (cacheBackDrop & 0x0F))

	void buildScreen();
	void buildSprite();
	void write_port_BE(u8 value);
	void write_port_BF(u8 value);

	u8 read_port_BE();
	u8 read_port_BF();

	bool updateScreenDim;

	bool commandSecond;
	u16 commandWord;
	u8 commandBuffer;
	u8 dataPort;
	u8 statusReg;

	u8 cgRam[32];
	u8 regs[0xA];
	
	u16 nameLatch;
	u16 tilePattern;
	u32 tileDefinition;
	u16 linePos;
	u8 cacheVScroll;
	u8 cacheHScroll;
	bool cacheDisplayActive;
	u8 cacheBackDrop;
	u16 patternOffset;
	u16 colAddress;

	struct {
		u8 spr;
		u8 bg;
		bool bgPrio;
	} pal[256];
	u16 palCount;

	struct {
		u8 count;
		bool cancel;
		u8 seekPos;
		int Line;
		u8* ptr;
		u8 height;
		u8 width;
		u8 pos[8];
		u8 tileLine[8];
	} spr;

	struct {
		bool detected; 
	} SprCol[256];

	void clearSprCol() {
		for(u16 i=0; i < 256; i++) {
			SprCol[i].detected = false;
		}
	}

	bool isSprCol( u16 pos ) {
		u8 helper = 12; // 0 - 12 left border
		if (pos < helper) return false;
		if (pos > (255+helper)) return false;
		return SprCol[(pos-helper) & 255].detected;
	}
	void setSprCol( u16 pos ) {
		SprCol[pos & 255].detected = true;
	}

	void refresh_screen_mode();
	void refreshScreenDimension();

	u16 vcounter;
	u16 hcounter;
	
	u8 borderBottom;
	u8 borderTop;
	u8 vblancLines;

	enum CpuAccess { none, readVram, writeVram, writeCgRam, writeReg };

	struct {
		u8 time;
		CpuAccess prepareJob;
		CpuAccess waitJob;
		u16 address;
		u8 data;
	} access;

	enum VMode { DISPLAY, BORDER_BOTTOM, BLANKING, BORDER_TOP } vmode;
	enum HMode { ACTIVE, RIGHT, HBLANKING, LEFT } hmode;

	void processH();

	u8 screenMode;
	u8 screenHeight;
	enum { MODE_0, MODE_1, MODE_2, MODE_1_2, MODE_3, MODE_1_3, MODE_2_3, MODE_1_2_3, MODE_4, MODE_4_224, MODE_4_240 };

	bool getBit(u8 reg, u8 bit) {
		return (reg >> bit) & 1;
	}

	void calcSprite_mode4(u8 pos) {
		_calcSprite_mode4(pos*2);
		_calcSprite_mode4(pos*2 + 1);
	}

	void prepareForMode4() {
		name_address_select = &Vdp::name_address_select_mode4;
		pattern_ram_load = &Vdp::pattern_ram_load_mode4;
		name_ram_load = &Vdp::name_ram_load_mode4;
		color_address_select = &Vdp::color_address_select_mode4;
		pattern_address_select = &Vdp::pattern_address_select_mode4;
		color_ram_load = &Vdp::color_ram_load_mode4;
		spritePreProcessing = &Vdp::spritePreProcessing_mode4;
		calcSprite = &Vdp::calcSprite_mode4;
		checkSpriteOverflow = &Vdp::checkSpriteOverflow_mode4;
		resetStatusReg = &Vdp::resetStatusReg_mode4;
		getBackdrop = &Vdp::getBackdrop_mode4;
		getMode = &Vdp::getMode4;
	}

	void prepareForMode2() {
		name_address_select = &Vdp::name_address_select_mode02;
		pattern_ram_load = &Vdp::pattern_ram_load_mode02;
		name_ram_load = &Vdp::name_ram_load_mode2;
		color_address_select = &Vdp::color_address_select_mode2;
		pattern_address_select = &Vdp::pattern_address_select_mode02;
		color_ram_load = &Vdp::color_ram_load_mode02;
		spritePreProcessing = &Vdp::spritePreProcessing_mode023;
		calcSprite = &Vdp::calcSprite_mode023;
		checkSpriteOverflow = &Vdp::checkSpriteOverflow_mode023;
		resetStatusReg = &Vdp::resetStatusReg_mode023;
		getBackdrop = &Vdp::getBackdrop_mode0123;
		getMode = &Vdp::getMode0123;
	}

	void prepareForMode0() {
		name_address_select = &Vdp::name_address_select_mode02;
		pattern_ram_load = &Vdp::pattern_ram_load_mode02;
		name_ram_load = &Vdp::name_ram_load_mode0;
		color_address_select = &Vdp::color_address_select_mode0;
		pattern_address_select = &Vdp::pattern_address_select_mode02;
		color_ram_load = &Vdp::color_ram_load_mode02;
		spritePreProcessing = &Vdp::spritePreProcessing_mode023;
		calcSprite = &Vdp::calcSprite_mode023;
		checkSpriteOverflow = &Vdp::checkSpriteOverflow_mode023;
		resetStatusReg = &Vdp::resetStatusReg_mode023;
		getBackdrop = &Vdp::getBackdrop_mode0123;
		getMode = &Vdp::getMode0123;
	}

	u8 getPixelFromTileLine(u8 pix, u32 srcLine);
	void calcSprite_mode023(u8 pos);
	void _calcSprite_mode4(u8 pos);
	void clearSprCache();
	void cpu_access();
	void handleCpuAccess();
	void updateDelay();
	void initDelay(CpuAccess newjob);
	
	u16 latchBgBase();
	void pattern_ram_load_mode02();
	void pattern_ram_load_mode4();
	void name_ram_load_mode0();
	void name_ram_load_mode2();
	void name_ram_load_mode4();
	void name_address_select_mode02(u8 col);
	void name_address_select_mode4(u8 col);
	void color_address_select_mode0();
	void color_address_select_mode2();
	void color_address_select_mode4();
	void pattern_address_select_mode02(u8 col);
	void pattern_address_select_mode4(u8 col);
	void color_ram_load_mode02(u8 col);
	void color_ram_load_mode4(u8 col);
	void checkSpriteOverflow_mode023();
	void checkSpriteOverflow_mode4();
	u8 resetStatusReg_mode023();
	u8 resetStatusReg_mode4();
	u8 getBackdrop_mode0123();
	u8 getBackdrop_mode4();
	Color::Mode getMode0123();
	Color::Mode getMode4();
	void drawLineOriginalTMSSprite(int xp, u8 drawLine, u8 color, bool zoomed);

	u16 getSprBase() {
		return ( regs[5] << 7) & 0x3F00;
	}
	u16 getSprBaseLegacy() {
		return ( regs[5] << 7) & 0x3F80;
	}

	void spritePreProcessing_mode4();
	void spritePreProcessing_mode023();
	void spriteInit();
	
	void addClocks(u8 cycles);
	void setLineBuffer();
	void frame();

	void calcFrameBorder();
	VMode getVMode( u16 _v );

	void incrementVCounter();

public:
	Vdp(_System* _sys, Bus& bus, Cpu& cpu);
	~Vdp() { free_mem(frameBuffer); }

	void enter();
	u8 mmio_read(u8 pos);
	void mmio_write(u8 pos, u8 value);

	void power();

	u8* output() {
		return frameBuffer;
	}

	void debugLook() {}
	u8 getInternalVCounter( u16 v );
	int getRealVCounter( u16 v );

	u16 getRenderHeight() {
		u16 out = lineBufferCounter;
		lineBufferCounter = 0;
		linePos = 0;
		return out-1;
	}
	unsigned get_frames_executed() { return frames_executed; }

	void setIrqState() {
		statusReg |= 0x80;
	}

	u8 getScreenHeight() {
		return screenHeight;
	}

	u8 getLineCounter() {
		return regs[0xA];
	}

	Color::Mode _getMode() {
		return (this->*getMode)();
	}
};

#endif;
